@@ -68,6 +68,17 @@ sub get_param_list {
    type => 't',
    default => 'localhost'
   },
+  {
+   name => 'smtp_username',
+   type => 't',
+   default => '',
+   checker => \&check_smtp_auth
+  },
+  {
+   name => 'smtp_password',
+   type => 'p',
+   default => ''
+  },
   {
    name => 'smtp_debug',
    type => 'b',
