@@ -94,6 +94,12 @@ sub REQUIRED_MODULES {
         module  => 'Email::MIME',
         version => '1.861'
     },
+    {
+        package => 'Email-MIME-Encodings',
+        module  => 'Email::MIME::Encodings',
+        # Fixes bug 486206
+        version => '1.313',
+    },
     {
         package => 'Email-MIME-Modifier',
         module  => 'Email::MIME::Modifier',
@@ -193,6 +199,8 @@ sub OPTIONAL_MODULES {
         package => 'SOAP-Lite',
         module  => 'SOAP::Lite',
         version => 0,
+        # These versions (0.70 -> 0.710.05) are affected by bug 468009
+        blacklist => ['^0\.70', '^0\.710?\.0[1-5]$'],
         feature => 'XML-RPC Interface'
     },
     {
@@ -232,19 +240,6 @@ sub OPTIONAL_MODULES {
     },
     );
 
-    # Even very new releases of perl (5.8.5) don't come with this version,
-    # so I didn't want to make it a general requirement just for
-    # running under mod_cgi.
-    # If Perl 5.10 is installed, then CGI 3.33 is already required. So this
-    # check is only relevant with Perl 5.8.x.
-    my $perl_ver = sprintf('%vd', $^V);
-    if (vers_cmp($perl_ver, '5.10') < 0) {
-        push(@modules, { package => 'CGI.pm',
-                         module  => 'CGI',
-                         version => '3.11',
-                         feature => 'mod_perl' });
-    }
-
     my $all_modules = _get_extension_requirements(
         'OPTIONAL_MODULES', \@modules);
     return $all_modules;
